@@ -13,16 +13,16 @@ use std::sync::Arc;
 const INDEX_FILES: [&str; 2] = ["index.html", "index.htm"];
 
 /// Request handler for files.
-pub fn file_handler(request: Request, state: Arc<AppState>, file: &str) -> Response {
+pub fn file_handler(request: Request, state: Arc<AppState>, file: &str, host: usize) -> Response {
     if let Some(response) = blacklist_check(&request, state.clone()) {
         return response;
     }
 
-    if let Some(response) = cache_check(&request, state.clone()) {
+    if let Some(response) = cache_check(&request, state.clone(), host) {
         return response;
     }
 
-    inner_file_handler(request, state, file.into())
+    inner_file_handler(request, state, file.into(), host)
 }
 
 /// Request handler for directories.
@@ -32,12 +32,13 @@ pub fn directory_handler(
     state: Arc<AppState>,
     directory: &str,
     matches: &str,
+    host: usize,
 ) -> Response {
     if let Some(response) = blacklist_check(&request, state.clone()) {
         return response;
     }
 
-    if let Some(response) = cache_check(&request, state.clone()) {
+    if let Some(response) = cache_check(&request, state.clone(), host) {
         return response;
     }
 
@@ -60,17 +61,15 @@ pub fn directory_handler(
                 ));
                 Response::empty(StatusCode::MovedPermanently)
                     .with_header(ResponseHeader::Location, format!("{}/", &request.uri))
-                    .with_request_compatibility(&request)
-                    .with_generated_headers()
             }
-            LocatedPath::File(path) => inner_file_handler(request, state, path),
+            LocatedPath::File(path) => inner_file_handler(request, state, path, host),
         }
     } else {
         state.logger.warn(&format!(
             "{}: 404 Not Found {}",
             request.address, request.uri
         ));
-        not_found(&request)
+        not_found()
     }
 }
 
@@ -86,11 +85,14 @@ pub fn redirect_handler(request: Request, state: Arc<AppState>, target: &str) ->
     ));
     Response::empty(StatusCode::MovedPermanently)
         .with_header(ResponseHeader::Location, target.into())
-        .with_request_compatibility(&request)
-        .with_generated_headers()
 }
 
-fn inner_file_handler(request: Request, state: Arc<AppState>, path: PathBuf) -> Response {
+fn inner_file_handler(
+    request: Request,
+    state: Arc<AppState>,
+    path: PathBuf,
+    host: usize,
+) -> Response {
     let file_extension = path.extension().map(|s| s.to_str().unwrap()).unwrap_or("");
 
     let mime_type = MimeType::from_extension(file_extension);
@@ -101,7 +103,7 @@ fn inner_file_handler(request: Request, state: Arc<AppState>, path: PathBuf) ->
 
     if state.config.cache.size_limit >= contents.len() {
         let mut cache = state.cache.write().unwrap();
-        cache.set(&request.uri, contents.clone(), mime_type);
+        cache.set(&request.uri, host, contents.clone(), mime_type);
         state.logger.debug(&format!("Cached route {}", request.uri));
     } else if state.config.cache.size_limit > 0 {
         state
@@ -115,8 +117,6 @@ fn inner_file_handler(request: Request, state: Arc<AppState>, path: PathBuf) ->
     Response::empty(StatusCode::OK)
         .with_header(ResponseHeader::ContentType, mime_type.into())
         .with_bytes(contents)
-        .with_request_compatibility(&request)
-        .with_generated_headers()
 }
 
 fn blacklist_check(request: &Request, state: Arc<AppState>) -> Option<Response> {
@@ -134,19 +134,17 @@ fn blacklist_check(request: &Request, state: Arc<AppState>) -> Option<Response>
         return Some(
             Response::empty(StatusCode::Forbidden)
                 .with_header(ResponseHeader::ContentType, "text/html".into())
-                .with_bytes(b"<h1>403 Forbidden</h1>")
-                .with_request_compatibility(request)
-                .with_generated_headers(),
+                .with_bytes(b"<h1>403 Forbidden</h1>"),
         );
     }
 
     None
 }
 
-fn cache_check(request: &Request, state: Arc<AppState>) -> Option<Response> {
+fn cache_check(request: &Request, state: Arc<AppState>, host: usize) -> Option<Response> {
     if state.config.cache.size_limit > 0 {
         let cache = state.cache.read().unwrap();
-        if let Some(cached) = cache.get(&request.uri) {
+        if let Some(cached) = cache.get(&request.uri, host) {
             state.logger.info(&format!(
                 "{}: 200 OK (cached) {}",
                 request.address, request.uri
@@ -154,9 +152,7 @@ fn cache_check(request: &Request, state: Arc<AppState>) -> Option<Response> {
             return Some(
                 Response::empty(StatusCode::OK)
                     .with_header(ResponseHeader::ContentType, cached.mime_type.into())
-                    .with_bytes(cached.data.clone())
-                    .with_request_compatibility(request)
-                    .with_generated_headers(),
+                    .with_bytes(cached.data.clone()),
             );
         }
         drop(cache);
@@ -165,10 +161,8 @@ fn cache_check(request: &Request, state: Arc<AppState>) -> Option<Response> {
     None
 }
 
-pub fn not_found(request: &Request) -> Response {
+pub fn not_found() -> Response {
     Response::empty(StatusCode::NotFound)
         .with_header(ResponseHeader::ContentType, "text/html".into())
         .with_bytes(b"<h1>404 Not Found</h1>")
-        .with_request_compatibility(request)
-        .with_generated_headers()
 }
